using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using System.Threading;
using System.Text.RegularExpressions;

/// <summary>
/// IrisPlacement Version v1.0
/// By Redeyes
/// Session plugin to quickly set Iris Placement Value on all persons
/// </summary>

namespace Redeyes{

    public class IrisPlacement : MVRScript
    {

        private JSONStorableFloat IrisPlacementValue;
        private UIDynamicSlider IrisPlacementSlider;
        private JSONStorable geometry;
        private DAZCharacterSelector character;
        private static DAZMorph IrisPlacement1;

        public override void Init()
        {
            try
            {
                SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
                SuperController.singleton.onAtomUIDsChangedHandlers += OnAtomUIDsChanged;
                
                if (containingAtom.type != "CoreControl" && containingAtom.type != "SessionPluginManager")
                {
                    SuperController.LogError($"Please load IrisPlacement as a Session or Scene Plugin, not with a '{containingAtom.type}' atom.");
                    DestroyImmediate(this);
                    return;
                }

                SetupInfoText(this, "<color=#606060><size=40><b>Iris Placement session Override</b></size></color>", 20.0f, false);

                IrisPlacementValue = new JSONStorableFloat("Iris Placement", 0f, IrisPlacementCallback, -1f, 1f, false);
                RegisterFloat(IrisPlacementValue);
                IrisPlacementSlider = CreateSlider(IrisPlacementValue, false);
                IrisPlacementSlider.rangeAdjustEnabled = true;
                IrisPlacementSlider.ConfigureQuickButtons(-0.001f, -0.01f, -0.1f, -1.0f, 0.001f, 0.01f, 0.1f, 1.0f);
                IrisPlacementSlider.valueFormat = "F3";
                SetupInfoText(this, "<color=#606060>This will override the Iris Placement morph on all persons in a scene\n" +
                "It will be applied on scene load and loading a person</color>", 150.0f, false);
            }
            catch (Exception e)
            {
                SuperController.LogError("Failed to initialize plugin: " + e);
            }
        }

        //Quick buttons
        public void IrisPlacementCallback(JSONStorableFloat IrisPlacementValue)
        {
            foreach (Atom atom in SuperController.singleton.GetAtoms())
            {
                if (atom.type == "Person")
                {
                    JSONStorable js = atom.GetStorableByID("geometry");
                    DAZCharacterSelector dcs = js as DAZCharacterSelector;
                    GenerateDAZMorphsControlUI morphUI = dcs.morphsControlUI;
                    if (morphUI != null)
                    {
                        IrisPlacement1 = morphUI.GetMorphByDisplayName("Iris Placement");
                        IrisPlacement1.morphValue = IrisPlacementValue.val;
                    }
                }
            }
        }

        private void OnSceneLoaded()
        {
            try {
                IrisPlacementCallback(IrisPlacementValue);
            }
            catch (Exception e) {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void OnAtomUIDsChanged(List<string> test)
        {
            try {
                 IrisPlacementCallback(IrisPlacementValue);
            }
            catch (Exception e) {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private static JSONStorableString SetupInfoText(MVRScript script, string text, float height, bool rightSide)
        {
            JSONStorableString storable = new JSONStorableString("Info", text);
            UIDynamic textfield = script.CreateTextField(storable, rightSide);
            textfield.height = height;
            return storable;
        }

    }
}
//yield WaitForSeconds(1);